import data_utility
import pairwise
import clustering_evaluation
import ksets, kswaps
import numpy as np
import random

# --------------- Input parameters -----------------
dataset_path = 'data_1200_200_32_5_1.txt'
output_labels_path_ksets = 'labels_ksets_1200_200_32_5_1.txt'
output_labels_path_kswaps = 'labels_kswaps_1200_200_32_5_1_kswaps.txt'
k = 32 # Number of clusters

np.random.seed(10)
random.seed(10)
        
# --- Read data
data = data_utility.read_file(dataset_path)

# --- Calclulate pairwise distances for evaluating the results
pw_dist = pairwise.calculate_pairwise_distance(data)

# --- K-sets clustering
print('Applying k-sets algoritm... ')
labels, representatives, sdh = ksets.cluster(data, k)
spd = clustering_evaluation.spd_distance(data, labels, np.copy(pw_dist))
spsd = clustering_evaluation.spd_distance(data, labels, np.copy(pw_dist), 2)
sdth = np.sum(sdh)
print('SPD=', spd, ' SPSD=', spsd, ' SDTH=', sdth, '\n')

data_utility.write_labels(labels, output_labels_path_ksets)

# --- K-swaps clustering
print('Applying k-swaps algorithm...')
labels, representatives, sdh = kswaps.cluster(data, k, 10)
spd = clustering_evaluation.spd_distance(data, labels, np.copy(pw_dist))
spsd = clustering_evaluation.spd_distance(data, labels, np.copy(pw_dist), 2)
sdth = np.sum(sdh)
print('SPD=', spd, ' SPSD=', spsd, ' SDTH=', sdth, '\n')

data_utility.write_labels(labels, output_labels_path_kswaps)

print('Process finished.')
        